#ifndef GFF3_H
#define GFF3_H

#ifdef __cplusplus
extern "C" {
#endif

#include <stdio.h>
#include <stdint.h>

typedef struct {
	char *tag;
	char *value;
} gff3_attr_t;

typedef struct {
	char    *seq_id;
	char    *source;
	char    *type;

	uint64_t start;
	uint64_t end;

	double   score;

	char     strand;
	int8_t   phase;

	uint16_t     attr_cnt;
	gff3_attr_t *attr_tbl;
} gff3_rec_t;

struct gff3_blk {
	struct gff3_blk *next;
	size_t           blk_size;
	size_t           blk_next;
	gff3_rec_t       blk_item[];
};

typedef struct {
	struct gff3_blk *blk0; /* first block */
	struct gff3_blk *blkN; /* last block */
	uint32_t         rcnt; /* total number of gff3 records */

	char       *path;
	FILE       *fp;
	int         wrote_hdr;
} gff3_t;

#define GFF3_DEFAULT_BLKSIZE 4096

gff3_t *gff3_new(const char *path);
gff3_rec_t *gff3_rec_new(void);
int gff3_rec_attr_add(gff3_rec_t *rec, char *tag, char *value);
int gff3_add_raw(gff3_t *g, gff3_rec_t *rec);
int gff3_write(gff3_t *g);
int gff3_free(gff3_t *g);

#ifdef __cplusplus
}
#endif

#endif /* GFF3_H */
